﻿// <copyright file="ProduseReader.cs" company="VEMA Calcul Electronic">
//   Copyright VEMA Calcul Electronic. All rights reserved.
// </copyright>
// <created>
//   <date value="2013.02.01" />
//   <author name="Emanuel Haisiuc" />
// </created>

using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using RO.Vema.ProduseShowroom.Entities;

namespace RO.Vema.ProduseShowroom.DataAccess.Interop
{
    public static class ProduseReader
    {
        private const string FisierProduse = "PRODUSE.DAT";
        private const string FisierCodBare = "CODBAR.DAT";

        private static List<Produs> _listProduse;
        private static Dictionary<decimal, Produs> _dictionaryCodProdusProduse;
        private static Dictionary<decimal, Produs> _dictionaryCodBareProduse;

        public static List<Produs> ListProduse
        {
            get
            {
                if(_listProduse == null)
                {
                    ConstruiesteListaProduse();
                }

                return _listProduse;
            }
        }

        public static Dictionary<decimal, Produs> DictionaryCodProdusProduse
        {
            get
            {
                if (_dictionaryCodProdusProduse == null)
                {
                    ConstruiesteListaProduse();
                }

                return _dictionaryCodProdusProduse;
            }
        }

        public static Dictionary<decimal, Produs> DictionaryCodBareProduse
        {
            get
            {
                if (_dictionaryCodBareProduse == null)
                {
                    if (_listProduse == null)
                    {
                        ConstruiesteListaProduse();
                    }

                    // ReSharper disable AssignNullToNotNullAttribute
                    _dictionaryCodBareProduse =
                        _listProduse.Where(p => p.CodBare != decimal.Zero).ToDictionary(p => p.CodBare);
                    // ReSharper restore AssignNullToNotNullAttribute
                }

                return _dictionaryCodBareProduse;
            }
        }

        private static void ConstruiesteListaProduse()
        {
            _listProduse = new List<Produs>();
            _dictionaryCodProdusProduse = new Dictionary<decimal, Produs>();

            // se citesc produsele
            using (FileStream sourceFile = new FileStream(FisierProduse, FileMode.Open, FileAccess.Read))
            {
                // sarim antetul de fisier
                sourceFile.Seek(128, SeekOrigin.Begin);

                using (BinaryReader br = new BinaryReader(sourceFile))
                {
                    while (br.PeekChar() != -1)
                    {
                        byte[] buffer = br.ReadBytes(140);

                        Produs prod = new Produs();
                        decimal codProdus;
                        if (!decimal.TryParse(Encoding.ASCII.GetString(buffer, 2, 11), out codProdus))
                        {
                            continue;
                        }

                        prod.Cod = codProdus;
                        prod.Denumire = Encoding.ASCII.GetString(buffer, 13, 35).Trim();
                        prod.NumarDesen = Encoding.ASCII.GetString(buffer, 48, 25).Trim();
                        prod.UnitateMasura = Encoding.ASCII.GetString(buffer, 76, 3).Trim();

                        _listProduse.Add(prod);
                        _dictionaryCodProdusProduse.Add(codProdus, prod);
                    }
                }
            }

            // se citesc codurile de bare
            using (FileStream sourceFile = new FileStream(FisierCodBare, FileMode.Open, FileAccess.Read))
            {
                // sarim antetul de fisier
                sourceFile.Seek(128, SeekOrigin.Begin);
                using (BinaryReader br = new BinaryReader(sourceFile))
                {
                    while (br.PeekChar() != -1)
                    {
                        byte[] buffer = br.ReadBytes(58);

                        decimal codProdus;
                        if (!decimal.TryParse(Encoding.ASCII.GetString(buffer, 2, 11), out codProdus))
                        {
                            continue;
                        }

                        Produs prod;
                        if (!_dictionaryCodProdusProduse.TryGetValue(codProdus, out prod))
                        {
                            continue;
                        }

                        decimal codBare;
                        if (!decimal.TryParse(Encoding.ASCII.GetString(buffer, 13, 13), out codBare))
                        {
                            continue;
                        }

                        prod.CodBare = codBare;
                    }
                }
            }

            _listProduse.Sort(ProdusSorterByCod);
        }

        private static int ProdusSorterByCod(Produs x, Produs y)
        {
            if (x == null && y == null)
            {
                return 0;
            }

            if (x == null)
            {
                return 1;
            }

            if (y == null)
            {
                return -1;
            }

            return x.Cod.CompareTo(y.Cod);
        }
    }
}