﻿// <copyright file="ProdusInShowroom.cs" company="VEMA Calcul Electronic">
//   Copyright VEMA Calcul Electronic. All rights reserved.
// </copyright>
// <created>
//   <date value="2013.02.01" />
//   <author name="Emanuel Haisiuc" />
// </created>

using System;
using System.ComponentModel;
using System.Globalization;
using System.Linq.Expressions;
using System.Reflection;

namespace RO.Vema.ProduseShowroom.Entities
{
    public class ProdusInShowroom
    {
        private decimal _codCorpIluminat;
        private decimal _codBec;
        private int _cantitateBecuri;
        private string _montatDe;
        private DateTime _dataMontarii;
        private string _demontatDe;
        private DateTime? _dataDemontarii;
        private string _comentarii;

        /// <summary>
        /// Event raised when a property changes.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        public decimal CodCorpIluminat
        {
            get
            {
                return _codCorpIluminat;
            }

            set
            {
                _codCorpIluminat = value;
                RaiseChanged(() => CodCorpIluminat);
            }
        }

        public decimal CodBec
        {
            get
            {
                return _codBec;
            }

            set
            {
                _codBec = value;
                RaiseChanged(() => CodBec);
            }
        }

        public int CantitateBecuri
        {
            get
            {
                return _cantitateBecuri;
            }

            set
            {
                _cantitateBecuri = value;
                RaiseChanged(() => CantitateBecuri);
            }
        }

        public string MontatDe
        {
            get
            {
                return _montatDe;
            }

            set
            {
                _montatDe = value;
                RaiseChanged(() => MontatDe);
            }
        }

        public DateTime DataMontarii
        {
            get
            {
                return _dataMontarii;
            }

            set
            {
                _dataMontarii = value;
                RaiseChanged(() => DataMontarii);
            }
        }

        public string DemontatDe
        {
            get
            {
                return _demontatDe;
            }

            set
            {
                _demontatDe = value;
                RaiseChanged(() => DemontatDe);
            }
        }

        public DateTime? DataDemontarii
        {
            get
            {
                return _dataDemontarii;
            }

            set
            {
                _dataDemontarii = value;
                RaiseChanged(() => DataDemontarii);
            }
        }

        public string Comentarii
        {
            get
            {
                return _comentarii;
            }

            set
            {
                    _comentarii = value;
                    RaiseChanged(() => Comentarii);
            }
        }

        public decimal DenumireCorpIluminat { get { return CodCorpIluminat; } }
        
        public decimal DenumireBec { get { return CodBec; } }

        protected bool Equals(ProdusInShowroom other)
        {
            return _codCorpIluminat == other._codCorpIluminat && _codBec == other._codBec &&
                   _cantitateBecuri == other._cantitateBecuri && string.Equals(_montatDe, other._montatDe) &&
                   _dataMontarii.Equals(other._dataMontarii) && string.Equals(_demontatDe, other._demontatDe) &&
                   _dataDemontarii.Equals(other._dataDemontarii) && string.Equals(_comentarii, other._comentarii);
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != this.GetType()) return false;
            return Equals((ProdusInShowroom) obj);
        }

        public override int GetHashCode()
        {
            unchecked
            {
                int hashCode = _codCorpIluminat.GetHashCode();
                hashCode = (hashCode * 397) ^ _codBec.GetHashCode();
                hashCode = (hashCode * 397) ^ _cantitateBecuri;
                hashCode = (hashCode * 397) ^ (_montatDe != null ? _montatDe.GetHashCode() : 0);
                hashCode = (hashCode * 397) ^ _dataMontarii.GetHashCode();
                hashCode = (hashCode * 397) ^ (_demontatDe != null ? _demontatDe.GetHashCode() : 0);
                hashCode = (hashCode * 397) ^ _dataDemontarii.GetHashCode();
                hashCode = (hashCode * 397) ^ (_comentarii != null ? _comentarii.GetHashCode() : 0);
                return hashCode;
            }
        }

        protected void RaiseChanged<TProperty>(Expression<Func<TProperty>> propertyExpression)
        {
            var property = propertyExpression.Body as MemberExpression;
            if (property == null || !(property.Member is PropertyInfo) || !IsPropertyOfThis(property))
            {
                throw new ArgumentException(
                    string.Format(
                        CultureInfo.CurrentCulture,
                        "Expression must be of the form 'this.PropertyName'. Invalid expression '{0}'.",
                        propertyExpression),
                    "propertyExpression");
            }

            RaisePropertyChangedEvent(property.Member.Name);
        }

        private bool IsPropertyOfThis(MemberExpression property)
        {
            var constant = RemoveCast(property.Expression) as ConstantExpression;
            return constant != null && constant.Value == this;
        }

        private Expression RemoveCast(Expression expression)
        {
            if (expression.NodeType == ExpressionType.Convert || expression.NodeType == ExpressionType.ConvertChecked)
            {
                return ((UnaryExpression)expression).Operand;
            }

            return expression;
        }

        /// <summary>
        /// Raises the PropertyChanged event.
        /// </summary>
        /// <param name="propertyName">the property's name</param>
        private void RaisePropertyChangedEvent(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}