﻿// <copyright file="FlexBindingSource.cs" company="VEMA Calcul Electronic">
//   Copyright VEMA Calcul Electronic. All rights reserved.
// </copyright>
// <created>
//   <date value="2010.07.07" />
//   <author name="Emanuel Haisiuc" />
// </created>

using System;
using System.Collections.Generic;
using System.Linq.Expressions;
using RO.Vema.Helper.BusinessObject.Services;

namespace RO.Vema.Helper.Client.DataBinding
{
    /// <summary>
    /// Clasa de binding source care elimina necesitatea de a folosi "magic strings" in data binding. 
    /// Se foloseste de linq expression tree.
    /// </summary>
    /// <typeparam name="TDataSource">tipul sursei de date cu care se face data binding</typeparam>
    public class FlexBusinessEntityPropertiesDisplayFilter<TDataSource>
    {
        /// <summary>
        /// items list
        /// </summary>
        private readonly List<FlexBusinessEntityPropertyDisplayInfo> _visibleProperties =
            new List<FlexBusinessEntityPropertyDisplayInfo>();

        public IList<FlexBusinessEntityPropertyDisplayInfo> PropertiesDisplayInfo
        {
            get { return _visibleProperties; }
        }

        /// <summary>
        /// Creates a DataBinding between a control property and a datasource property
        /// </summary>
        /// <param name="datasourceMemberAccesor">A lambda expression which specifies the datasource property (something like datasource => datasource.Property) </param>
        /// <param name="width">desired property width</param>
        /// <returns>the instance of the  binding created</returns>
        public void SetVisibleBusinessEntityProperty(Expression<Func<TDataSource, object>> datasourceMemberAccesor, int width)
        {
            string sourcePropertyName = FlexReflector.GetPropertyName(datasourceMemberAccesor);

            if (!_visibleProperties.Exists(propertyInfo => propertyInfo.PropertyName == sourcePropertyName))
            {
                _visibleProperties.Add(
                    new FlexBusinessEntityPropertyDisplayInfo(sourcePropertyName, _visibleProperties.Count, width));
            }
        }

        /// <summary>
        /// Verifica daca coloana indicata este vizibila.
        /// <para>
        /// Cheia coloanei este aceiasi cu numele proprietatii.
        /// </para>
        /// </summary>
        /// <param name="columnKey">cheia coloanei</param>
        /// <returns>indica daca este afisabila sau nu</returns>
        public FlexBusinessEntityPropertyDisplayInfo GetBusinessEntityPropertyDisplayInfo(string columnKey)
        {
            return _visibleProperties.Find(propertyInfo => propertyInfo.PropertyName == columnKey);
        }
    }
}