// <copyright file="TypeHelper.cs" company="VEMA Calcul Electronic">
//   Copyright VEMA Calcul Electronic. All rights reserved.
// </copyright>
// <created>
//   <date value="2007.11.16" />
//   <author name="Emanuel Haisiuc" />
// </created>

using System;

namespace RO.Vema.Helper.BusinessObject
{
    /// <summary>
    /// Clasa helper care determina valorile implicite pentru un tip de data indicat.
    /// </summary>
    [Serializable]
    public class TypeHelper
    {
        /// <summary>
        /// Metoda generica de determinat valoarea implicita a tipului de data indicat de <see cref="T"/>.
        /// </summary>
        /// <typeparam name="T">tipul de data pentru care se cauta valoarea implicita</typeparam>
        /// <param name="defaultValue">parametru out in care se va gasi valoarea implicita</param>
        /// <returns>
        /// true daca tipul de data este cunoscut si valoarea implicita a fost determinata, false altfel
        /// </returns>
        public bool GetDefaultValue<T>(out T defaultValue)
        {
            object temp;
            bool success = GetDefaultValue(typeof(T), out temp);
            if (success)
            {
                defaultValue = (T)temp;
            }
            else
            {
                defaultValue = default(T);
            }

            return success;
        }

        /// <summary>
        /// Metoda determina valoarea implicita a tipului de data indicat.
        /// </summary>
        /// <param name="tip">tipul pentru care se cauta valoarea implicita</param>
        /// <param name="defaultValue">parametru out in care se va gasi valoarea implicita</param>
        /// <returns>
        /// true daca tipul de data este cunoscut si valoarea implicita a fost determinata, false altfel
        /// </returns>
        public virtual bool GetDefaultValue(Type tip, out object defaultValue)
        {
            if (tip == typeof(Guid))
            {
                defaultValue = Guid.Empty;
                return true;
            }

            if (tip == typeof(Guid?))
            {
                defaultValue = null;
                return true;
            }

            if (tip == typeof(int))
            {
                defaultValue = 0;
                return true;
            }

            if (tip == typeof(int?))
            {
                defaultValue = null;
                return true;
            }

            if (tip == typeof(decimal))
            {
                defaultValue = decimal.Zero;
                return true;
            }

            if (tip == typeof(short))
            {
                defaultValue = (short)0;
                return true;
            }

            if (tip == typeof(string))
            {
                defaultValue = string.Empty;
                return true;
            }

            if (tip == typeof(DateTime))
            {
                defaultValue = DateTime.MinValue;
                return true;
            }

            if (tip == typeof(DateTime?))
            {
                defaultValue = null;
                return true;
            }

            if (tip == typeof(uint))
            {
                defaultValue = (uint)0;
                return true;
            }

            if (tip == typeof(bool))
            {
                defaultValue = false;
                return true;
            }

            if (tip == typeof(bool?))
            {
                defaultValue = new bool?(false);
                return true;
            }

            if (tip == typeof(short))
            {
                defaultValue = (short)0;
                return true;
            }

            if (tip == typeof(byte))
            {
                defaultValue = (byte)0;
                return true;
            }

            defaultValue = null;
            return false;
        }
    }
}