﻿// <copyright file="ProdusInShowroomView.cs" company="VEMA Calcul Electronic">
//   Copyright VEMA Calcul Electronic. All rights reserved.
// </copyright>
// <created>
//   <date value="2013.02.04" />
//   <author name="Emanuel Haisiuc" />
// </created>

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Windows.Forms;
using RO.Vema.Helper.Client;
using RO.Vema.Helper.Client.DataBinding;
using RO.Vema.ProduseShowroom.Entities;

namespace RO.Vema.ProduseShowroom
{
    public partial class ProdusInShowroomViewLinear : Form
    {
        private ProdusInShowroom _produsInShowroom;
        private readonly MainView _parentForm;
        private DateTime _startTime;
        private BusinessEntityToControlsLinker<ProdusInShowroom> _linker;

        public ProdusInShowroomViewLinear(MainView parentForm)
        {
            _parentForm = parentForm;

            InitializeComponent();

            SetLinks();
        }

        public ProdusInShowroom ProdusInShowroom
        {
            set
            {
                _produsInShowroom = value;
                _produsInShowroomBindingSource.DataSource = _produsInShowroom;
                ValidateChildren();
            }
        }

        public List<Produs> Produse
        {
            set
            {
                List<Produs> produseView = new List<Produs>();
                produseView.Add(new Produs {Denumire = "-- selectati --"});
                produseView.AddRange(value);

                _produsCorpIluminatBindingSource.DataSource = produseView;
                _produsBecBindingSource.DataSource = produseView;
            }
        }

        protected override void OnLoad(EventArgs e)
        {
            _startTime = DateTime.Now;

            base.OnLoad(e);
            
            btnSalveaza.Click += OnSalvazaClick;
            btnRenunta.Click += OnRenuntaClick;
        }

        protected override void OnShown(EventArgs e)
        {
            base.OnShown(e);

            DateTime endTime = DateTime.Now;
            _parentForm.AddMessage(string.Format("Showing the child form took {0} ms", endTime.Subtract(_startTime).TotalMilliseconds));
            _parentForm.DisplayTotalTimeTookToShow(endTime);
        }

        private void OnSalvazaClick(object sender, EventArgs e)
        {
            //SQLiteRepository.Save(_produsInShowroom);
            DialogResult = DialogResult.OK;
            Close();
        }

        private void OnRenuntaClick(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void SetLinks()
        {
            _linker = new BusinessEntityToControlsLinker<ProdusInShowroom>(_produsInShowroomBindingSource, _errorProvider);

            _linker.CreateLink(cboCodCorpIluminat, ctrl => ctrl.Value, data => data.CodCorpIluminat);
            _linker.CreateLink(cboDenumireCorpIluminat, ctrl => ctrl.Value, data => data.CodCorpIluminat);
            _linker.CreateLink(cboCodBec, ctrl => ctrl.Value, data => data.CodBec);
            _linker.CreateLink(cboDenumireBec, ctrl => ctrl.Value, data => data.CodBec);

            Binding cantitateBecuriBind = _linker.CreateLink(
                numCantitateBecuri, ctrl => ctrl.Text, data => data.CantitateBecuri);
            cantitateBecuriBind.Format += NumericDataBindingHelper.OnIntegerNumericDataBindingFormat;
            cantitateBecuriBind.Parse += NumericDataBindingHelper.OnIntegerNumericDataBindingParse;

            _linker.CreateLink(txtMontatDe, ctrl => ctrl.Text, data => data.MontatDe);
            _linker.CreateLink(dteDataMontarii, ctrl => ctrl.Text, data => data.DataMontarii);
            _linker.CreateLink(txtDemontatDe, ctrl => ctrl.Text, data => data.DemontatDe);

            Binding dataDemontariiBind = _linker.CreateLink(dteDataDemontarii, ctrl => ctrl.Text, data => data.DataDemontarii);
            dataDemontariiBind.Parse += OnDataDemontariiParse;

            _linker.CreateLink(txtComentarii, ctrl => ctrl.Text, data => data.Comentarii);
        }

        private void OnDataDemontariiParse(object sender, ConvertEventArgs e)
        {
            if (!(e.Value is string))
            {
                return;
            }
            
            string stringValue = (string) e.Value;

            if (string.IsNullOrEmpty(stringValue))
            {
                e.Value = null;
            }
            else if (stringValue ==
                     string.Format(
                         "{0}{0}{1}{0}{0}{1}{0}{0}{0}{0}",
                         dteDataDemontarii.PromptChar,
                         CultureInfo.CurrentCulture.DateTimeFormat.DateSeparator))
            {
                e.Value = null;
            }
        }
    }
}
